//
//  CXAudioPlayer.h
//  Chroniton
//
//  Created by Logan Collins on 4/8/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>


@class CXBackgroundAudio, CXSound;


/*!
 * @class CXAudioPlayer
 * @abstract Coordinates playing background audio and sound effects
 */
@interface CXAudioPlayer : NSObject

/*!
 * @method sharedPlayer
 * @abstract The shared player object
 * 
 * @result A CXAudioPlayer object
 */
+ (CXAudioPlayer *)sharedPlayer;


/*!
 * @property backgroundAudio
 * @abstract The current background audio
 * 
 * @result A CXBackgroundAudio object
 */
@property (strong) CXBackgroundAudio *backgroundAudio;


/*!
 * @method playSound:
 * @abstract Plays a sound effect
 * 
 * @param sound
 * The sound effect to play
 */
- (void)playSound:(CXSound *)sound;
- (void)stopPlayingSound:(CXSound *)sound;


/*!
 * @property paused
 * @abstract Whether audio playback is currently paused
 * 
 * @result A BOOL value
 */
@property (readonly, getter=isPaused) BOOL paused;

/*!
 * @method pause
 * @abstract Pauses all audio playback
 */
- (void)pause;

/*!
 * @method resume
 * @abstract Resumes paused audio playback
 */
- (void)resume;

@end
