//
//  CXAudioPlayer.m
//  Chroniton
//
//  Created by Logan Collins on 4/8/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXAudioPlayer.h"
#import "CXBackgroundAudio.h"
#import "CXSound.h"


@implementation CXAudioPlayer {
	AVPlayer *_backgroundAudioPlayer;
	
	NSMutableArray *_players;
	NSMutableDictionary *_soundsToPlayers;
	
	BOOL _paused;
}

@synthesize backgroundAudio=_backgroundAudio;

+ (CXAudioPlayer *)sharedPlayer {
	static CXAudioPlayer *sharedPlayer = nil;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		sharedPlayer = [[self alloc] init];
	});
	return sharedPlayer;
}

- (id)init {
	self = [super init];
	if (self) {
		_players = [NSMutableArray array];
		_soundsToPlayers = [NSMutableDictionary dictionary];
		
		[self addObserver:self forKeyPath:@"backgroundAudio" options:(NSKeyValueObservingOptionNew) context:nil];
	}
	return self;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    if (object == self && [keyPath isEqualToString:@"backgroundAudio"]) {
		CXBackgroundAudio *newAudio = self.backgroundAudio;
		AVAsset *newAsset = [newAudio asset];
		
		if (_backgroundAudioPlayer != nil && newAudio == nil) {
			// Stop
			[[NSNotificationCenter defaultCenter] removeObserver:self name:AVPlayerItemDidPlayToEndTimeNotification object:[_backgroundAudioPlayer currentItem]];
			
			[_backgroundAudioPlayer pause];
			_backgroundAudioPlayer = nil;
		}
		else if (_backgroundAudioPlayer != nil && newAudio != nil) {
			// Replace
			[[NSNotificationCenter defaultCenter] removeObserver:self name:AVPlayerItemDidPlayToEndTimeNotification object:[_backgroundAudioPlayer currentItem]];
			
			AVPlayerItem *item = [AVPlayerItem playerItemWithAsset:newAsset];
			[_backgroundAudioPlayer replaceCurrentItemWithPlayerItem:item];
			
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(backgroundAudioDidPlayToEndTime:) name:AVPlayerItemDidPlayToEndTimeNotification object:item];
		}
		else {
			// Play
			AVPlayerItem *item = [AVPlayerItem playerItemWithAsset:newAsset];
			_backgroundAudioPlayer = [AVPlayer playerWithPlayerItem:item];
			_backgroundAudioPlayer.actionAtItemEnd = AVPlayerActionAtItemEndNone;
			[_backgroundAudioPlayer play];
			
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(backgroundAudioDidPlayToEndTime:) name:AVPlayerItemDidPlayToEndTimeNotification object:item];
		}
    }
	else [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
}


#pragma mark -
#pragma mark Playback

- (void)backgroundAudioDidPlayToEndTime:(NSNotification *)notification {
	AVPlayerItem *item = [notification object];
	[item seekToTime:[_backgroundAudio loopTime]];
}

- (void)playSound:(CXSound *)sound {
	AVAsset *asset = [sound asset];
	AVPlayerItem *playerItem = [AVPlayerItem playerItemWithAsset:asset];
	AVPlayer *player = [AVPlayer playerWithPlayerItem:playerItem];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(soundDidPlayToEndTime:) name:AVPlayerItemDidPlayToEndTimeNotification object:playerItem];
	
	NSNumber *key = [NSNumber numberWithInteger:[sound hash]];
	[_players addObject:player];
	[_soundsToPlayers setObject:player forKey:key];
	
	[player play];
}

- (void)stopPlayingSound:(CXSound *)sound {
	NSNumber *key = [NSNumber numberWithInteger:[sound hash]];
	AVPlayer *player = [_soundsToPlayers objectForKey:key];
	if (player != nil) {
		[player pause];
		[_players removeObject:player];
		[_soundsToPlayers removeObjectForKey:key];
	}
}

- (void)soundDidPlayToEndTime:(NSNotification *)notification {
	AVPlayerItem *item = [notification object];
	
}

- (BOOL)isPaused {
	return _paused;
}

- (void)pause {
	if (_paused) {
		return;
	}
	
	[self willChangeValueForKey:@"paused"];
	_paused = YES;
	
	
	[self didChangeValueForKey:@"paused"];
}

- (void)resume {
	if (!_paused) {
		return;
	}
	
	[self willChangeValueForKey:@"paused"];
	_paused = NO;
	
	
	[self didChangeValueForKey:@"paused"];
}

@end
