//
//  CXBackgroundAudio.h
//  Chroniton
//
//  Created by Logan Collins on 4/8/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>


/*!
 * @class CXBackgroundAudio
 * @abstract Long-running background audio track
 */
@interface CXBackgroundAudio : NSObject

/*!
 * @method backgroundAudioWithURL:
 * @abstract Create a new background audio track from a URL
 * 
 * @param URL
 * The URL of the audio source
 * 
 * @result A CXBackgroundAudio object
 */
+ (CXBackgroundAudio *)backgroundAudioWithURL:(NSURL *)URL;

/*!
 * @method backgroundAudioWithURL:loopTime:
 * @abstract Create a new background audio track from a URL
 * 
 * @param URL
 * The URL of the audio source
 * 
 * @param loopTime
 * The time index to loop back to when the audio finishes
 * 
 * @result A CXBackgroundAudio object
 */
+ (CXBackgroundAudio *)backgroundAudioWithURL:(NSURL *)URL loopTime:(CMTime)loopTime;

/*!
 * @method backgroundAudioWithAsset:
 * @abstract Create a new background audio track from an AVFoundation asset
 * 
 * @param asset
 * The asset representing the audio source
 * 
 * @result A CXBackgroundAudio object
 */
+ (CXBackgroundAudio *)backgroundAudioWithAsset:(AVAsset *)asset;

/*!
 * @method backgroundAudioWithAsset:loopTime:
 * @abstract Create a new background audio track from an AVFoundation asset
 * 
 * @param asset
 * The asset representing the audio source
 * 
 * @param loopTime
 * The time index to loop back to when the audio finishes
 * 
 * @result A CXBackgroundAudio object
 */
+ (CXBackgroundAudio *)backgroundAudioWithAsset:(AVAsset *)asset loopTime:(CMTime)loopTime;


/*!
 * @property URL
 * @abstract The URL of the audio
 * 
 * @result An NSURL object
 */
@property (copy, readonly) NSURL *URL;

/*!
 * @property asset
 * @abstract The AVFoundation asset representation of the audio
 * 
 * @result An NSURL object
 */
@property (strong, readonly) AVAsset *asset;

/*!
 * @property loopTime
 * @abstract The time index to loop back to when the audio finishes
 * 
 * @result An NSURL object
 */
@property (readonly) CMTime loopTime;

@end
