//
//  CXSound.h
//  Chroniton
//
//  Created by Logan Collins on 4/10/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>


/*!
 * @class CXSound
 * @abstract Short-running sound effect
 */
@interface CXSound : NSObject

/*!
 * @method soundWithURL:
 * @abstract Create a new sound from a URL
 * 
 * @param URL
 * The URL of the sound
 * 
 * @result A CXSound object
 */
+ (CXSound *)soundWithURL:(NSURL *)URL;

/*!
 * @method soundWithAsset:
 * @abstract Create a new sound from an AVFoundation asset
 * 
 * @param asset
 * The asset representation of the sound
 * 
 * @result A CXSound object
 */
+ (CXSound *)soundWithAsset:(AVAsset *)asset;

/*!
 * @property URL
 * @abstract The URL of the sound
 * 
 * @result An NSURL object
 */
@property (copy, readonly) NSURL *URL;

/*!
 * @property asset
 * @abstract The AVFoundation asset representation of the sound
 * 
 * @result An NSURL object
 */
@property (strong, readonly) AVAsset *asset;

@end
