//
//  CXDataStore.h
//  Chroniton
//
//  Created by Logan Collins on 4/20/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>


/*!
 * @class CXDataStore
 * @abstract A key-value coding compliant container object for persistent data storage
 * 
 * @discussion
 * CXData store is thread-safe.
 * 
 * All values within CXDataStore must be property list compatible. Therefore, the only
 * allowed object types are: NSString, NSNumber, NSData, NSDate, NSArray and NSDictionary.
 */
@interface CXDataStore : NSObject

+ (CXDataStore *)dataStore;
+ (CXDataStore *)dataStoreWithContentsOfURL:(NSURL *)aURL error:(NSError **)outError;

- (BOOL)writeToURL:(NSURL *)aURL error:(NSError **)outError;

- (id)objectForKey:(NSString *)key;
- (void)setObject:(id)value forKey:(NSString *)key;

- (NSArray *)arrayForKey:(NSString *)key;
- (void)setArray:(NSArray *)array forKey:(NSString *)key;

- (NSData *)dataForKey:(NSString *)key;
- (void)setData:(NSData *)value forKey:(NSString *)key;

- (NSDictionary *)dictionaryForKey:(NSString *)key;
- (void)setDictionary:(NSDictionary *)dictionary forKey:(NSString *)key;

- (NSString *)stringForKey:(NSString *)key;
- (void)setString:(NSString *)value forKey:(NSString *)key;

- (BOOL)boolForKey:(NSString *)key;
- (void)setBool:(BOOL)value forKey:(NSString *)key;

- (NSInteger)integerForKey:(NSString *)key;
- (void)setInteger:(NSInteger)value forKey:(NSString *)key;

- (float)floatForKey:(NSString *)key;
- (void)setFloat:(float)value forKey:(NSString *)key;

- (double)doubleForKey:(NSString *)key;
- (void)setDouble:(double)value forKey:(NSString *)key;

- (void)removeObjectForKey:(NSString *)key;

- (NSDictionary *)dictionaryRepresentation;

@end
