//
//  CXDevice.m
//  Chroniton
//
//  Created by Logan Collins on 4/27/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXDevice.h"

#if TARGET_OS_IPHONE
#import <UIKit/UIKit.h>
#endif


@implementation CXDevice

static CXDevice *sharedDevice = nil;

+ (CXDevice *)currentDevice {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedDevice = [[self alloc] init];
    });
    return sharedDevice;
}

+ (id)allocWithZone:(NSZone *)zone {
    if (sharedDevice != nil) {
        return nil;
    }
    if (self != [CXDevice class]) {
        @throw [NSException exceptionWithName:NSInternalInconsistencyException reason:@"CXDevice should not be subclassed." userInfo:nil];
    }
    return [super allocWithZone:zone];
}

- (id)init {
    self = [super init];
    if (self) {
#if TARGET_OS_IPHONE
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(uiDeviceBatteryStateDidChange:) name:UIDeviceBatteryStateDidChangeNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(uiDeviceBatteryLevelDidChange:) name:UIDeviceBatteryLevelDidChangeNotification object:nil];
#endif
    }
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}


#pragma mark -
#pragma mark User Interface

- (CXUserInterfaceIdiom)userInterfaceIdiom {
#if TARGET_OS_IPHONE
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return CXUserInterfaceIdiomPhone;
    }
    else {
        return CXUserInterfaceIdiomPad;
    }
#elif TARGET_OS_MAC
    return CXUserInterfaceIdiomMac;
#endif
}


#pragma mark -
#pragma mark Battery

- (void)uiDeviceBatteryStateDidChange:(NSNotification *)notification {
    [[NSNotificationCenter defaultCenter] postNotificationName:CXDeviceBatteryStateDidChangeNotification object:self];
}

- (void)uiDeviceBatteryLevelDidChange:(NSNotification *)notification {
    [[NSNotificationCenter defaultCenter] postNotificationName:CXDeviceBatteryLevelDidChangeNotification object:self];
}

- (BOOL)isBatteryStateSupported {
#if TARGET_OS_IPHONE
    return YES;
#elif TARGET_OS_MAC
    return NO;
#endif
}

- (BOOL)isBatteryMonitoringEnabled {
#if TARGET_OS_IPHONE
    return [[UIDevice currentDevice] isBatteryMonitoringEnabled];
#elif TARGET_OS_MAC
    return NO;
#endif
}

- (void)setBatteryMonitoringEnabled:(BOOL)batteryMonitoringEnabled {
#if TARGET_OS_IPHONE
    [[UIDevice currentDevice] setBatteryMonitoringEnabled:batteryMonitoringEnabled];
#endif
}

- (CXDeviceBatteryState)batteryState {
#if TARGET_OS_IPHONE
    UIDeviceBatteryState batteryState = [[UIDevice currentDevice] batteryState];
    switch (batteryState) {
        case UIDeviceBatteryStateUnplugged: return CXDeviceBatteryStateUnplugged;
        case UIDeviceBatteryStateCharging: return CXDeviceBatteryStateCharging;
        case UIDeviceBatteryStateFull: return CXDeviceBatteryStateFull;
        default: break;
    }
    return CXDeviceBatteryStateUnknown;
#elif TARGET_OS_MAC
    return CXDeviceBatteryStateUnknown;
#endif
}

- (float)batteryLevel {
#if TARGET_OS_IPHONE
    return [[UIDevice currentDevice] batteryLevel];
#elif TARGET_OS_MAC
    return 1.0;
#endif
}

@end


NSString * const CXDeviceBatteryLevelDidChangeNotification = @"CXDeviceBatteryLevelDidChangeNotification";
NSString * const CXDeviceBatteryStateDidChangeNotification = @"CXDeviceBatteryStateDidChangeNotification";

