//
//  CXTimer.h
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>


@class CXDirector;


/*!
 * @class CXTimer
 * @abstract A timer object tied to the directed scheduler
 * 
 * @discussion
 * A CXTimer object is backed by an NSTimer object under the hood, but is properly
 * paused and resumed when its parent director object is paused or resumed.
 */
@interface CXTimer : NSObject

/*!
 * @property director
 * @abstract The director that controls the timer
 * 
 * @result A CXDirector object
 */
@property (weak, readonly) CXDirector *director;

/*!
 * @property interval
 * @abstract The timer interval
 * 
 * @result An NSTimeInterval value
 */
@property (readonly) NSTimeInterval interval;

/*!
 * @property repeats
 * @abstract Whether the timer is rescheduled automatically after firing
 * 
 * @result A BOOL value
 */
@property (readonly) BOOL repeats;

/*!
 * @property block
 * @abstract The block invoked when the timer fires
 * 
 * @result A block object
 */
@property (copy, readonly) void (^block)(void);


/*!
 * @method invalidate
 * @abstract Cancels the timer
 * 
 * @discussion
 * If the timer is non-repeating and has already fired, this method has no effect.
 */
- (void)invalidate;

@end
