//
//  CXTouch.h
//  Chroniton
//
//  Created by Logan Collins on 4/22/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <QuartzCore/QuartzCore.h>
#if TARGET_OS_IPHONE
#import <CoreGraphics/CoreGraphics.h>
#endif


enum {
    CXTouchPhaseBegan,
    CXTouchPhaseMoved,
    CXTouchPhaseStationary,
    CXTouchPhaseEnded,
    CXTouchPhaseCancelled,
};
typedef NSUInteger CXTouchPhase;


/*!
 * @class CXTouch
 * @abstract Represents a touch
 */
@interface CXTouch : NSObject

/*!
 * @property location
 * @abstract The location of the touch
 * 
 * @result A CGPoint value
 */
@property (readonly) CGPoint location;

/*!
 * @method locationInLayer:
 * @abstract The location of the touch in a specific layer
 * 
 * @param layer
 * The layer
 * 
 * @result A CGPoint value
 */
- (CGPoint)locationInLayer:(CALayer *)layer;

/*!
 * @property previousLocation
 * @abstract The previous location of the touch
 * 
 * @result A CGPoint value
 */
@property (readonly) CGPoint previousLocation;

/*!
 * @method previousLocationInLayer:
 * @abstract The previous location of the touch in a specific layer
 * 
 * @param layer
 * The layer
 * 
 * @result A CGPoint value
 */
- (CGPoint)previousLocationInLayer:(CALayer *)layer;

/*!
 * @property tapCount
 * @abstract The number of taps
 * 
 * @result An NSUInteger value
 */
@property (readonly) NSUInteger tapCount;

/*!
 * @property timestamp
 * @abstract The timestamp of the touch
 * 
 * @result An NSTimeInterval value
 */
@property (readonly) NSTimeInterval timestamp;

/*!
 * @property phase
 * @abstract The phase of the touch
 * 
 * @result A CXTouchPhase value
 */
@property (readonly) CXTouchPhase phase;

@end
