//
//  CXImage.m
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXImage.h"
#import "CXGraphics.h"


static CFMutableDictionaryRef _namedImagesDictionary = NULL;


@interface CXImage ()

@property (copy) NSString *name;

@end


@implementation CXImage {
    CGImageRef _cgImage;
}

@synthesize name=_name;

+ (CXImage *)imageNamed:(NSString *)name {
    if (_namedImagesDictionary == NULL) {
        // The entire collection should be weak
        CFDictionaryValueCallBacks valueCallBacks = { 0, NULL, NULL, NULL, CFEqual };
        _namedImagesDictionary = CFDictionaryCreateMutable(NULL, 0, &kCFCopyStringDictionaryKeyCallBacks, &valueCallBacks);
    }
    
    CXImage *image = [(__bridge NSMutableDictionary *)_namedImagesDictionary objectForKey:name];
    
    if (image == nil) {
        NSURL *URL = [[NSBundle mainBundle] URLForResource:name withExtension:@"png"];
        if (URL != nil) {
            image = [self imageWithContentsOfURL:URL];
            image.name = name;
        }
    }
    
    if (image != nil) {
        [(__bridge NSMutableDictionary *)_namedImagesDictionary setObject:image forKey:name];
    }
    
    return image;
}

+ (CXImage *)imageWithContentsOfURL:(NSURL *)URL {
    return [self imageWithContentsOfFile:[URL path]];
}

+ (CXImage *)imageWithContentsOfFile:(NSString *)file {
    CGDataProviderRef source = CGDataProviderCreateWithFilename([file UTF8String]);
    CGImageRef cgImage = CGImageCreateWithPNGDataProvider(source, NULL, NO, kCGRenderingIntentDefault);
    CXImage *image = [[self alloc] initWithCGImage:cgImage];
    CGImageRelease(cgImage);
    return image;
}

+ (CXImage *)imageWithCGImage:(CGImageRef)cgImage {
    return [[self alloc] initWithCGImage:cgImage];
}

+ (CXImage *)imageWithCIImage:(CIImage *)ciImage {
    CIContext *context = nil;
#if TARGET_OS_IPHONE
    context = [CIContext contextWithOptions:nil];
#elif TARGET_OS_MAC
    context = [CIContext contextWithCGContext:CXGraphicsGetCurrentContext() options:nil];
#endif
    CGImageRef cgImage = [context createCGImage:ciImage fromRect:[ciImage extent]];
    CXImage *image = [self imageWithCGImage:cgImage];
    CGImageRelease(cgImage);
    return image;
}

- (id)initWithCGImage:(CGImageRef)cgImage {
	if (cgImage == NULL) {
		return nil;
	}
	
	self = [super init];
	if (self) {
		_cgImage = (CGImageRef)CFRetain(cgImage);
	}
	return self;
}

- (void)dealloc {
    NSString *name = self.name;
    if (name != nil) {
        [(__bridge NSMutableDictionary *)_namedImagesDictionary removeObjectForKey:name];
    }
    
    if (_cgImage != NULL) {
        CGImageRelease(_cgImage);
    }
}


#pragma mark -
#pragma mark Accessors

- (CGSize)size {
    return CGSizeMake(CGImageGetWidth(_cgImage), CGImageGetHeight(_cgImage));
}

- (CGImageRef)CGImage {
    return _cgImage;
}

- (CIImage *)CIImage {
	return [CIImage imageWithCGImage:_cgImage];
}


#pragma mark -
#pragma mark Drawing

- (void)drawInRect:(CGRect)rect {
	[self drawInRect:rect blendMode:kCGBlendModeNormal alpha:1.0];
}

- (void)drawInRect:(CGRect)rect blendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha {
	CGContextRef c = CXGraphicsGetCurrentContext();
	CGContextSaveGState(c);
	CGContextSetBlendMode(c, blendMode);
	CGContextSetAlpha(c, alpha);
	CGContextDrawImage(c, rect, _cgImage);
	CGContextRestoreGState(c);
}

@end
