//
//  CXSprite.h
//  Chroniton
//
//  Created by Logan Collins on 4/7/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Chroniton/CXImage.h>


/*!
 * @class CXSprite
 * @abstract Represents a sprite image
 */
@interface CXSprite : CXImage

/*!
 * @method spriteWithImage:
 * @abstract Creates a new sprite from a subregion of an image
 * 
 * @param image
 * The image from which to create
 * 
 * @param frame
 * The subregion of the image from which to create the sprite
 * 
 * @result A CXSprite object
 */
+ (CXSprite *)spriteWithImage:(CXImage *)image frame:(CGRect)frame;


/*!
 * @property image
 * @abstract The image backing the sprite
 * 
 * @result A CXImage object
 */
@property (strong, readonly) CXImage *image;

/*!
 * @property frame
 * @abstract The sprite's region of the image
 * 
 * @result A CGRect value
 */
@property (readonly) CGRect frame;

@end


/*!
 * @category CXImage(CXSpriteAdditions)
 * @abstract Additions to CXImage for sprite atlas behaviors
 */
@interface CXImage (CXSpriteAdditions)

/*!
 * @method spriteWithFrame:
 * @abstract Creates a new sprite from a subregion of the image
 * 
 * @param frame
 * The frame rectangle of the sprite
 * 
 * @result A CXSprite object
 */
- (CXSprite *)spriteWithFrame:(CGRect)frame;

@end
