//
//  CXSpriteLayer.m
//  Chroniton
//
//  Created by Logan Collins on 4/16/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXSpriteLayer.h"
#import "CXSprite.h"
#import "CXImage.h"


@interface CXSpriteLayer ()

@property (copy) NSArray *sprites;
@property NSUInteger spriteIndex;

@end


@implementation CXSpriteLayer

@synthesize sprites=_sprites;
@synthesize spriteIndex=_spriteIndex;

+ (id <CAAction>)defaultActionForKey:(NSString *)aKey {
	if ([aKey isEqualToString:@"contents"]
        || [aKey isEqualToString:@"spriteIndex"]) {
		return (id <CAAction>)[NSNull null];
	}
	return [super defaultActionForKey:aKey];
}

+ (BOOL)needsDisplayForKey:(NSString *)key {
    if ([key isEqualToString:@"spriteIndex"]) {
		return YES;
	}
	return [super needsDisplayForKey:key];
}

- (id)init {
    self = [super init];
    if (self) {
        self.magnificationFilter = kCAFilterNearest;
    }
    return self;
}

- (void)display {
	if (self.sprites != nil) {
        NSUInteger spriteIndex = [[self presentationLayer] spriteIndex];
		CXSprite *sprite = [self.sprites objectAtIndex:spriteIndex];
		self.contents = (id)sprite.CGImage;
	}
	else {
		[super display];
	}
}

- (void)setContentsFromSprite:(CXSprite *)sprite {
	[self removeAnimationForKey:@"CXSpriteAnimation"];
	self.sprites = nil;
	self.spriteIndex = 0;
	
	[CATransaction begin];
	[CATransaction setAnimationDuration:0.0];
	
	self.contents = (id)sprite.CGImage;
	
	[CATransaction commit];
}

- (void)setContentsFromSpritesInArray:(NSArray *)sprites duration:(CFTimeInterval)duration {
	[self removeAnimationForKey:@"CXSpriteAnimation"];
    
    if ([sprites count] == 1) {
        [self setContentsFromSprite:[sprites objectAtIndex:0]];
        return;
    }
	
	self.sprites = sprites;
    if (self.spriteIndex >= [sprites count]) {
        self.spriteIndex = 0;
    }
	
	NSMutableArray *values = [NSMutableArray arrayWithCapacity:[sprites count]+1];
	NSMutableArray *keyTimes = [NSMutableArray arrayWithCapacity:[sprites count]+1];
	NSMutableArray *timingFunctions = [NSMutableArray arrayWithCapacity:[sprites count]];
	CAMediaTimingFunction *timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionDefault];
	
	CGFloat step = 1.0 / (CGFloat)[sprites count];
	for (NSUInteger i=0; i<[sprites count]; i++) {
		CGFloat keyTime = step * (CGFloat)i;
		
		[values addObject:[NSNumber numberWithInteger:i]];
		[keyTimes addObject:[NSNumber numberWithDouble:keyTime]];
		
		[timingFunctions addObject:timingFunction];
	}
	
	[values addObject:[NSNumber numberWithInteger:0]];
	[keyTimes addObject:[NSNumber numberWithDouble:1.0]];
	
	CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"spriteIndex"];
	animation.values = values;
	animation.keyTimes = keyTimes;
	animation.timingFunctions = timingFunctions;
	animation.calculationMode = kCAAnimationDiscrete;
	animation.repeatCount = HUGE_VALF;
	animation.duration = duration * [sprites count];
	
	[self addAnimation:animation forKey:@"CXSpriteAnimation"];
}

@end
