//
//  CXMapTable.m
//  Chroniton
//
//  Created by Logan Collins on 4/22/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXMapTable.h"


// 
// !IMPORTANT!:
// CXMapTable is excepted from ARC in its compiler settings for weak-reference reasons.
// 


static const void * CXObjectRetain(CFAllocatorRef allocator, const void * value) {
    return (const void *)[(id)value retain];
}

static void CXObjectRelease(CFAllocatorRef allocator, const void * value) {
    [(id)value release];
}

static CFStringRef CXObjectCopyDescription(const void * value) {
    return (CFStringRef)[[(id)value description] copy];
}

static Boolean CXObjectEqual(const void * value1, const void * value2) {
    return (Boolean)[(id)value1 isEqual:(id)value2];
}

static CFHashCode CXObjectHash(const void * value) {
    return (CFHashCode)[(id)value hash];
}


@implementation CXMapTable {
    CFMutableDictionaryRef _dictionary;
}

+ (CXMapTable *)mapTableWithStrongToStrongObjects {
    CFDictionaryKeyCallBacks keyCallBacks = { 0, CXObjectRetain, CXObjectRelease, CXObjectCopyDescription, CXObjectEqual, CXObjectHash };
    CFDictionaryValueCallBacks valueCallBacks = { 0, CXObjectRetain, CXObjectRelease, CXObjectCopyDescription, CXObjectEqual };
    CFMutableDictionaryRef dictionary = CFDictionaryCreateMutable(NULL, 0, &keyCallBacks, &valueCallBacks);
    return [[self alloc] initWithOwnedDictionary:dictionary];
}

+ (CXMapTable *)mapTableWithStrongToUnretainedObjects {
    CFDictionaryKeyCallBacks keyCallBacks = { 0, CXObjectRetain, CXObjectRelease, CXObjectCopyDescription, CXObjectEqual, CXObjectHash };
    CFDictionaryValueCallBacks valueCallBacks = { 0, NULL, NULL, CXObjectCopyDescription, CXObjectEqual };
    CFMutableDictionaryRef dictionary = CFDictionaryCreateMutable(NULL, 0, &keyCallBacks, &valueCallBacks);
    return [[self alloc] initWithOwnedDictionary:dictionary];
}

+ (CXMapTable *)mapTableWithUnretainedToStrongObjects {
    CFDictionaryKeyCallBacks keyCallBacks = { 0, NULL, NULL, CXObjectCopyDescription, CXObjectEqual, CXObjectHash };
    CFDictionaryValueCallBacks valueCallBacks = { 0, CXObjectRetain, CXObjectRelease, CXObjectCopyDescription, CXObjectEqual };
    CFMutableDictionaryRef dictionary = CFDictionaryCreateMutable(NULL, 0, &keyCallBacks, &valueCallBacks);
    return [[self alloc] initWithOwnedDictionary:dictionary];
}

+ (CXMapTable *)mapTableWithUnretainedToUnretainedObjects {
    CFDictionaryKeyCallBacks keyCallBacks = { 0, NULL, NULL, CXObjectCopyDescription, CXObjectEqual, CXObjectHash };
    CFDictionaryValueCallBacks valueCallBacks = { 0, NULL, NULL, CXObjectCopyDescription, CXObjectEqual };
    CFMutableDictionaryRef dictionary = CFDictionaryCreateMutable(NULL, 0, &keyCallBacks, &valueCallBacks);
    return [[self alloc] initWithOwnedDictionary:dictionary];
}

- (id)initWithOwnedDictionary:(CFMutableDictionaryRef)dictionary {
    self = [super init];
    if (self) {
        _dictionary = dictionary;
    }
    return self;
}

- (void)dealloc {
    CFRelease(_dictionary);
    [super dealloc];
}

- (NSUInteger)countByEnumeratingWithState:(NSFastEnumerationState *)state objects:(id __unsafe_unretained [])buffer count:(NSUInteger)len {
    return [(NSMutableDictionary *)_dictionary countByEnumeratingWithState:state objects:buffer count:len];
}

- (NSEnumerator *)keyEnumerator {
    return [(NSMutableDictionary *)_dictionary keyEnumerator];
}

- (NSEnumerator *)objectEnumerator {
    return [(NSMutableDictionary *)_dictionary objectEnumerator];
}

- (NSUInteger)count {
    return CFDictionaryGetCount(_dictionary);
}

- (NSArray *)allKeys {
    return [(NSMutableDictionary *)_dictionary allKeys];
}

- (NSArray *)allObjects {
    return [(NSMutableDictionary *)_dictionary allValues];
}

- (id)objectForKey:(id)key {
    return (id)CFDictionaryGetValue(_dictionary, (const void *)key);
}

- (void)setObject:(id)value forKey:(id)key {
    if (value == nil) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Value cannot be nil" userInfo:nil];
    }
    if (key == nil) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Key cannot be nil" userInfo:nil];
    }
    CFDictionarySetValue(_dictionary, (const void *)key, (const void *)value);
}

- (void)removeObjectForKey:(id)key {
    if (key == nil) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@"Key cannot be nil" userInfo:nil];
    }
    CFDictionaryRemoveValue(_dictionary, (const void *)key);
}

- (void)removeAllObjects {
    CFDictionaryRemoveAllValues(_dictionary);
}

@end
