//
//  CXTiledLayer.m
//  Chroniton
//
//  Created by Logan Collins on 4/16/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXTiledLayer.h"
#import "CXTile.h"

#import "CXGraphics.h"


@interface CXTiledLayer ()

- (CGPoint)pointForPosition:(CXTilePosition)position;
- (CXTilePosition)positionAtPoint:(CGPoint)point;

- (CALayer *)dequeueTileLayer;
- (void)recycleTileLayer:(CALayer *)layer;

@end


@implementation CXTiledLayer {
	NSMutableSet *_tileLayers;
	NSMutableDictionary *_tileLayersByPosition;
	NSMutableSet *_recycledTileLayers;
}

@synthesize contentProvider=_contentProvider;
@synthesize tileSize=_tileSize;

+ (BOOL)needsDisplayForKey:(NSString *)key {
	if ([key isEqualToString:@"tileSize"]) {
		return YES;
	}
	return [super needsDisplayForKey:key];
}

+ (id)defaultActionForKey:(NSString *)key {
	if ([key isEqualToString:@"bounds"]) {
		return nil;
	}
	return [super defaultActionForKey:key];
}

- (id)init {
	self = [super init];
	if (self) {
		_tileLayers = [NSMutableSet set];
		_tileLayersByPosition = [NSMutableDictionary dictionary];
		_recycledTileLayers = [NSMutableSet set];
	}
	return self;
}

- (void)display {
	self.bounds = CGRectMake(0.0, 0.0, self.tileSize.width, self.tileSize.height);
	
	CGContextRef c = CXGraphicsGetCurrentContext();
	CGRect boundingRect = CGContextGetClipBoundingBox(c);
	
	CXTilePosition originPosition = [self positionAtPoint:self.bounds.origin];
	if (originPosition.x > 0) {
		originPosition.x = originPosition.x - 1;
	}
	if (originPosition.y > 0) {
		originPosition.y = originPosition.y - 1;
	}
	
	NSUInteger tilesWide = (boundingRect.size.width / self.tileSize.width) + 2;
	NSUInteger tilesHigh = (boundingRect.size.height / self.tileSize.height) + 2;
	NSUInteger numTiles = (tilesWide * tilesHigh);
	
	for (NSUInteger i = originPosition.x; i < originPosition.x + tilesWide; i++) {
		for (NSUInteger j = originPosition.y; j < originPosition.y + tilesWide; j++) {
			CXTilePosition position = CXTilePositionMake(i, j);
			CXTile *tile = [[self contentProvider] tiledLayer:self tileAtPosition:position];
			if (tile != nil) {
				
			}
		}
	}
}


#pragma mark -
#pragma mark Tiles

- (CGPoint)pointForPosition:(CXTilePosition)position {
	return CGPointMake(position.x * self.tileSize.width, position.y * self.tileSize.height);
}

- (CXTilePosition)positionAtPoint:(CGPoint)point {
	return CXTilePositionMake((NSUInteger)floorf(0.5f + point.x / self.tileSize.width), (NSUInteger)floorf(0.5f + point.y / self.tileSize.height));
}

- (CALayer *)dequeueTileLayer {
	CALayer *layer = [_recycledTileLayers anyObject];
	if (layer != nil) {
		[_recycledTileLayers removeObject:layer];
	}
	else {
		layer = [CALayer layer];
	}
	return layer;
}

- (void)recycleTileLayer:(CALayer *)layer {
	for (CALayer *sublayer in [[layer sublayers] copy]) {
		[sublayer removeFromSuperlayer];
	}
	[layer removeFromSuperlayer];
	[_recycledTileLayers addObject:layer];
}

@end
