//
//  CXGameView.m
//  Chroniton
//
//  Created by Logan Collins on 4/16/12.
//  Copyright (c) 2012 Sunflower Softworks. All rights reserved.
//

#import "CXGameView.h"
#import "CXDirector_Private.h"
#import "CXEvent_Private.h"
#import "CXTouch_Private.h"
#import "CXMapTable.h"


@implementation CXGameView {
    CXMapTable *_UITouchesToCXTouches;
}

@synthesize director=_director;

#if TARGET_OS_IPHONE
- (id)initWithFrame:(CGRect)frameRect {
#elif TARGET_OS_MAC
- (id)initWithFrame:(NSRect)frameRect {
#endif
	self = [super initWithFrame:frameRect];
	if (self) {
        _UITouchesToCXTouches = [CXMapTable mapTableWithStrongToStrongObjects];
        
		[self setDirector:[CXDirector sharedDirector]];
	}
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
	self = [super initWithCoder:aDecoder];
	if (self) {
        _UITouchesToCXTouches = [CXMapTable mapTableWithStrongToStrongObjects];
        
		[self setDirector:[CXDirector sharedDirector]];
	}
	return self;
}


#pragma mark -
#pragma mark NSResponder methods

#if !TARGET_OS_IPHONE

- (BOOL)acceptsFirstResponder {
	return YES;
}

- (BOOL)becomeFirstResponder {
	return YES;
}

- (BOOL)resignFirstResponder {
	return YES;
}

- (void)keyDown:(NSEvent *)theEvent {
	CXEvent *event = [[CXEvent alloc] init];
	event.type = CXEventTypeKeyboard;
	event.subtype = CXEventSubtypeKeyDown;
    event.timestamp = [theEvent timestamp];
	event.keyCode = [theEvent keyCode];
	event.repeat = [theEvent isARepeat];
	event.nsEvent = theEvent;
	
	[self.director handle_keyDown:event];
}

- (void)keyUp:(NSEvent *)theEvent {
	CXEvent *event = [[CXEvent alloc] init];
	event.type = CXEventTypeKeyboard;
	event.subtype = CXEventSubtypeKeyUp;
    event.timestamp = [theEvent timestamp];
	event.keyCode = [theEvent keyCode];
	event.repeat = [theEvent isARepeat];
	event.nsEvent = theEvent;
	
	[self.director handle_keyUp:event];
}

- (void)mouseDown:(NSEvent *)theEvent {
	NSPoint thePoint = [self convertPointToLayer:[self convertPointFromBase:[theEvent locationInWindow]]];
	
	CXEvent *event = [[CXEvent alloc] init];
	event.type = CXEventTypeMouse;
    event.timestamp = [theEvent timestamp];
	event.location = NSPointToCGPoint(thePoint);
	if ([event type] == NSLeftMouseDown) {
		event.subtype = CXEventSubtypeLeftMouseDown;
	}
	else if ([event type] == NSRightMouseDown) {
		event.subtype = CXEventSubtypeRightMouseDown;
	}
	event.nsEvent = theEvent;
	
	[self.director handle_mouseDown:event];
}

- (void)mouseUp:(NSEvent *)theEvent {
	NSPoint thePoint = [self convertPointToLayer:[self convertPointFromBase:[theEvent locationInWindow]]];
	
	CXEvent *event = [[CXEvent alloc] init];
	event.type = CXEventTypeMouse;
    event.timestamp = [theEvent timestamp];
	event.location = NSPointToCGPoint(thePoint);
	if ([event type] == NSLeftMouseDown) {
		event.subtype = CXEventSubtypeLeftMouseUp;
	}
	else if ([event type] == NSRightMouseDown) {
		event.subtype = CXEventSubtypeRightMouseUp;
	}
	event.nsEvent = theEvent;
	
	[self.director handle_mouseUp:event];
}

- (void)mouseDragged:(NSEvent *)theEvent {
	NSPoint thePoint = [self convertPointToLayer:[self convertPointFromBase:[theEvent locationInWindow]]];
	
	CXEvent *event = [[CXEvent alloc] init];
	event.type = CXEventTypeMouse;
    event.timestamp = [theEvent timestamp];
	event.location = NSPointToCGPoint(thePoint);
	if ([event type] == NSLeftMouseDown) {
		event.subtype = CXEventSubtypeLeftMouseDragged;
	}
	else if ([event type] == NSRightMouseDown) {
		event.subtype = CXEventSubtypeRightMouseDragged;
	}
	event.nsEvent = theEvent;
	
	[self.director handle_mouseDragged:event];
}

#endif


#pragma mark -
#pragma mark UIResponder methods

#if TARGET_OS_IPHONE

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	CXEvent *cxEvent = [[CXEvent alloc] init];
    cxEvent.type = CXEventTypeTouches;
    cxEvent.timestamp = [event timestamp];
    
    NSMutableSet *cxTouches = [NSMutableSet setWithCapacity:[touches count]];
    for (UITouch *touch in touches) {
        CXTouch *cxTouch = [[CXTouch alloc] init];
        
        cxTouch.director = self.director;
        cxTouch.location = [touch locationInView:self];
        cxTouch.previousLocation = [touch previousLocationInView:self];
        cxTouch.timestamp = [touch timestamp];
        cxTouch.tapCount = [touch tapCount];
        cxTouch.phase = [touch phase];
        
        [_UITouchesToCXTouches setObject:cxTouch forKey:touch];
        [cxTouches addObject:cxTouch];
    }
    [cxEvent setTouches:cxTouches];
    
    [self.director handle_touchesBegan:cxTouches withEvent:cxEvent];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	CXEvent *cxEvent = [[CXEvent alloc] init];
    cxEvent.type = CXEventTypeTouches;
    cxEvent.timestamp = [event timestamp];
    
    NSMutableSet *cxTouches = [NSMutableSet setWithCapacity:[touches count]];
    for (UITouch *touch in touches) {
        CXTouch *cxTouch = [_UITouchesToCXTouches objectForKey:touch];
        if (cxTouch == nil) {
            cxTouch = [[CXTouch alloc] init];
            [_UITouchesToCXTouches setObject:cxTouch forKey:touch];
        }
        
        cxTouch.director = self.director;
        cxTouch.location = [touch locationInView:self];
        cxTouch.previousLocation = [touch previousLocationInView:self];
        cxTouch.timestamp = [touch timestamp];
        cxTouch.tapCount = [touch tapCount];
        cxTouch.phase = [touch phase];
        [cxTouches addObject:cxTouch];
    }
    [cxEvent setTouches:cxTouches];
    
    [self.director handle_touchesMoved:cxTouches withEvent:cxEvent];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	CXEvent *cxEvent = [[CXEvent alloc] init];
    cxEvent.type = CXEventTypeTouches;
    cxEvent.timestamp = [event timestamp];
    
    NSMutableSet *cxTouches = [NSMutableSet setWithCapacity:[touches count]];
    for (UITouch *touch in touches) {
        CXTouch *cxTouch = [_UITouchesToCXTouches objectForKey:touch];
        if (cxTouch == nil) {
            cxTouch = [[CXTouch alloc] init];
        }
        
        cxTouch.director = self.director;
        cxTouch.location = [touch locationInView:self];
        cxTouch.previousLocation = [touch previousLocationInView:self];
        cxTouch.timestamp = [touch timestamp];
        cxTouch.tapCount = [touch tapCount];
        cxTouch.phase = [touch phase];
        [cxTouches addObject:cxTouch];
    }
    [cxEvent setTouches:cxTouches];
    
    [self.director handle_touchesEnded:cxTouches withEvent:cxEvent];
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
	CXEvent *cxEvent = [[CXEvent alloc] init];
    cxEvent.type = CXEventTypeTouches;
    cxEvent.timestamp = [event timestamp];
    
    NSMutableSet *cxTouches = [NSMutableSet setWithCapacity:[touches count]];
    for (UITouch *touch in touches) {
        CXTouch *cxTouch = [_UITouchesToCXTouches objectForKey:touch];
        if (cxTouch == nil) {
            cxTouch = [[CXTouch alloc] init];
        }
        
        cxTouch.director = self.director;
        cxTouch.location = [touch locationInView:self];
        cxTouch.previousLocation = [touch previousLocationInView:self];
        cxTouch.timestamp = [touch timestamp];
        cxTouch.tapCount = [touch tapCount];
        cxTouch.phase = [touch phase];
        [cxTouches addObject:cxTouch];
    }
    [cxEvent setTouches:cxTouches];
    
    [self.director handle_touchesCancelled:cxTouches withEvent:cxEvent];
}

#endif

@end
